# -*- coding: utf-8 -*-
#
# Author: Ingelrest François (Athropos@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import gobject, gtk

from tools            import prefsManager
from ctrlPanel        import CtrlPanel
from gstPlayer        import GSTPlayer
from statusbar        import Statusbar
from tracklist        import Tracklist
from threading        import Lock
from notification     import Notification
from restoreTracklist import RestoreTracklist


# List of messages that can be sent/received by modules
# A message is always associated with a dictionnary that contains its parameters
# The dictionnary may be empty if there's no parameter
# When nothing is specified, there is no parameter
(
    # Commands: may be sent when an action is requested
    MSG_CMD_PLAY,             # Play a file -- filename
    MSG_CMD_TOGGLE_PAUSE,     # Switch between play/pause
    MSG_CMD_STOP,             # Stop playing the current track, if any
    MSG_CMD_SEEK,             # Jump to a position -- seconds
    MSG_CMD_NEXT,             # Jump to the next track
    MSG_CMD_PREVIOUS,         # Jump to the previous track
    MSG_CMD_TRACKLIST_SET,    # Replace the tracklist -- files, playNow, sortTracks
    MSG_CMD_TRACKLIST_ADD,    # Append files to the tracklist -- files, sortTracks
    MSG_CMD_TRACKLIST_CLEAR,  # Clear the tracklist

    # Events: sent whenever an event occurs
    MSG_EVT_PAUSED,           # The current track has been paused
    MSG_EVT_UNPAUSED,         # The current track has been unpaused
    MSG_EVT_STOPPED,          # The current track has been stopped
    MSG_EVT_NEW_TRACK,        # The current track has changed -- trackInfo, hasNext, hasPrevious
    MSG_EVT_APP_STARTED,      # The application has just started
    MSG_EVT_APP_QUIT,         # The application is quitting
    MSG_EVT_NEW_TRACKLIST,    # A new tracklist has been set -- files, playtime
    MSG_EVT_TRACK_ENDED,      # The current track has ended
    MSG_EVT_TRACK_POSITION,   # The track position has changed -- seconds

    # End value
    MSG_END_VALUE
) = range(19)


# For each message, store the list of registered modules
mHandlers = {}
for msg in range(MSG_END_VALUE):
    mHandlers[msg] = []

# All registered modules
mModules = []

# This mutex protects all the functions from concurrent acces
mMutex = Lock()


def load(wTree):
    """
        Instanciate the modules
        The 'wTree' parameter is the tree of widgets returned by Glade
        The list is static for the moment, module will eventually be loaded dynamically one day
    """
    mModules.append(GSTPlayer(wTree))
    mModules.append(CtrlPanel(wTree))
    mModules.append(Tracklist(wTree))
    mModules.append(Statusbar(wTree))
    mModules.append(Notification(wTree))
    mModules.append(RestoreTracklist(wTree))

    for module in mModules:
        module.start()


def register(module, msgList):
    """
        Register the given module for all messages in the given list
        If a module is already registered for a message, nothing happens
    """
    mMutex.acquire()
    for msg in msgList:
        if module not in mHandlers[msg]:
            mHandlers[msg].append(module)
    mMutex.release()


def unregister(module):
    """
        Unregister the given module for all messages
    """
    mMutex.acquire()
    for handlers in mHandlers.values():
        if module in handlers:
            handlers.remove(module)
    mMutex.release()


def __postMsg(msg, params):
    """
        This is the 'real' postMsg() function
        It MUST be executed in the main GTK loop
    """
    for module in mHandlers[msg]:
        module.postMsg(msg, params)


def postMsg(msg, params={}):
    """
        Post a message in the queue of the modules
        Only registered modules for this message will receive it
    """
    # We need to ensure that posting messages will be done by the main GTK thread
    # Otherwise, the code of non-threaded modules will be executed in the caller's thread, which may not be the main GTK thread
    # This can cause problems when such modules call GTK functions
    mMutex.acquire()
    gobject.idle_add(__postMsg, msg, params)
    mMutex.release()


def __postQuitMsg():
    """
        This is the 'real' postQuitMsg() function
        It MUST be executed in the main GTK loop
    """
    __postMsg(MSG_EVT_APP_QUIT, {})

    for module in mModules:
        module.join()

    prefsManager.save()
    gtk.main_quit()


def postQuitMsg():
    """
        Post a MSG_EVT_APP_QUIT in each module's queue and exit the application
        Preferences are saved before exiting
    """
    # As with postMsg(), we need to ensure that the code will be executed by the main GTK thread
    mMutex.acquire()
    gobject.idle_add(__postQuitMsg)
    mMutex.release()
