#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Ingelrest François (Athropos@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import gtk, modules, tools, tracklist

from modules.module import Module


class CtrlPanel(Module):
    """
        This module manages the control panel with the buttons and the slider
    """


    def __init__(self, wTree):
        """
            Constructor
        """
        Module.__init__(self)
        self.currTrackLength = 0;
        # Retrieve widgets
        self.btnStop          = wTree.get_widget('btn-stop')
        self.btnPlay          = wTree.get_widget('btn-play')
        self.btnNext          = wTree.get_widget('btn-next')
        self.btnPrevious      = wTree.get_widget('btn-previous')
        self.sclPosition      = wTree.get_widget('scl-position')
        self.lblElapsedTime   = wTree.get_widget('lbl-elapsedTime')
        self.lblRemainingTime = wTree.get_widget('lbl-remainingTime')
        # At first, the tracklist is empty
        self.btnStop.set_sensitive(False)
        self.btnPlay.set_sensitive(False)
        self.btnNext.set_sensitive(False)
        self.sclPosition.set_sensitive(False)
        self.btnPrevious.set_sensitive(False)
        self.lblElapsedTime.hide()
        self.lblRemainingTime.hide()
        # Messages handled by this module
        modules.register(self, [modules.MSG_EVT_STOPPED, modules.MSG_EVT_PAUSED, modules.MSG_EVT_UNPAUSED, modules.MSG_EVT_NEW_TRACK, modules.MSG_EVT_NEW_TRACKLIST, modules.MSG_EVT_TRACK_POSITION])
        # GTK handlers
        wTree.signal_connect('onBtnStop',          lambda widget: modules.postMsg(modules.MSG_CMD_STOP))
        wTree.signal_connect('onBtnPlay',          lambda widget: modules.postMsg(modules.MSG_CMD_TOGGLE_PAUSE))
        wTree.signal_connect('onBtnNext',          lambda widget: modules.postMsg(modules.MSG_CMD_NEXT))
        wTree.signal_connect('onBtnPrevious',      lambda widget: modules.postMsg(modules.MSG_CMD_PREVIOUS))
        wTree.signal_connect('onSclTrackPosition', lambda range, scroll, value: modules.postMsg(modules.MSG_CMD_SEEK, {'seconds': int(value)}))


    def newTracklist(self, count):
        """
            Enable/disable widgets based on the new tracklist
        """
        if count == 0:
            self.btnPlay.set_sensitive(False)
        else:
            self.btnPlay.set_sensitive(True)

        self.btnStop.set_sensitive(False)
        self.btnNext.set_sensitive(False)
        self.btnPrevious.set_sensitive(False)


    def newTrack(self, trackInfo, hasPrevious, hasNext):
        """
            A new track is being played
        """
        # Enable/disable widgets
        self.btnNext.set_sensitive(hasNext)
        self.btnPrevious.set_sensitive(hasPrevious)
        self.btnStop.set_sensitive(True)
        self.btnPlay.set_sensitive(True)
        self.sclPosition.set_sensitive(True)
        # Change the default action of the play button
        self.btnPlay.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON))
        # Update labels and track position
        self.currTrackLength = trackInfo[tracklist.LENGTH]
        self.lblElapsedTime.set_label(tools.sec2str(0))
        self.lblRemainingTime.set_label(tools.sec2str(self.currTrackLength))
        self.lblElapsedTime.show()
        self.lblRemainingTime.show()
        self.sclPosition.set_value(0)
        self.sclPosition.set_range(0, self.currTrackLength)


    def stopped(self):
        """
            Stop playing
        """
        # Enable/disable widgets
        self.btnStop.set_sensitive(False)
        self.btnNext.set_sensitive(False)
        self.btnPrevious.set_sensitive(False)
        self.sclPosition.set_sensitive(False)
        # Change the default action of the play button
        self.btnPlay.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_BUTTON))
        # Update labels and track position
        self.lblElapsedTime.hide()
        self.lblRemainingTime.hide()
        self.sclPosition.set_value(0)


    def paused(self):
        """
            Paused
        """
        self.btnPlay.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_BUTTON))
        self.sclPosition.set_sensitive(False)


    def unpaused(self):
        """
            Unpaused
        """
        self.btnPlay.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON))
        self.sclPosition.set_sensitive(True)


    def trackPosition(self, elapsedTime):
        """
            The track position has changed
        """
        # Sometimes, the real track length may be higher than the expected one
        if elapsedTime >= self.currTrackLength:
            self.lblRemainingTime.set_label(tools.sec2str(0))
        else:
            self.lblRemainingTime.set_label(tools.sec2str(self.currTrackLength - elapsedTime))
        self.lblElapsedTime.set_label(tools.sec2str(elapsedTime))
        self.sclPosition.set_value(elapsedTime)


   # --== Message handler ==--


    def handleMsg(self, msg, params):
        """
            A message has been received
        """
        if   msg == modules.MSG_EVT_PAUSED:         self.paused()
        elif msg == modules.MSG_EVT_STOPPED:        self.stopped()
        elif msg == modules.MSG_EVT_UNPAUSED:       self.unpaused()
        elif msg == modules.MSG_EVT_NEW_TRACK:      self.newTrack(params['trackInfo'], params['hasPrevious'], params['hasNext'])
        elif msg == modules.MSG_EVT_NEW_TRACKLIST:  self.newTracklist(len(params['files']))
        elif msg == modules.MSG_EVT_TRACK_POSITION: self.trackPosition(params['seconds'])
