# -*- coding: utf-8 -*-
#
# Author: Ingelrest François (Athropos@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import pygst
pygst.require('0.10')

import gobject, gst, modules, os.path

from tools          import consts
from modules.module import Module


class GSTPlayer(Module):
    """
        This module is the 'real' GStreamer player
    """


    def __init__(self, wTree):
        """
            Constructor
        """
        Module.__init__(self)
        self.player = gst.element_factory_make('playbin', consts.appName)
        self.bus    = self.player.get_bus()
        self.timer  = None
        self.bus.add_signal_watch()
        self.bus.connect('message', self.onGSTMsg)
        # Messages handled by this module
        modules.register(self, [modules.MSG_CMD_PLAY, modules.MSG_CMD_STOP, modules.MSG_CMD_TOGGLE_PAUSE, modules.MSG_CMD_SEEK])


    def __timerFunc(self):
        """
            Generate a message with the current track position
        """
        if self.player.get_state()[1] == gst.STATE_PLAYING:
            seconds = int(self.player.query_position(gst.FORMAT_TIME)[0] / 1000000000)
            modules.postMsg(modules.MSG_EVT_TRACK_POSITION, {'seconds': seconds})
        return True


    def __startTimer(self):
        """
            Create the timer
        """
        self.timer = gobject.timeout_add(500, self.__timerFunc)


    def __stopTimer(self):
        """
            Stop the timer if needed
        """
        if self.timer is not None:
            gobject.source_remove(self.timer)
            self.timer = None


    def onGSTMsg(self, bus, msg):
        """
            GStreamer message handler
        """
        if msg.type == gst.MESSAGE_EOS or msg.type == gst.MESSAGE_ERROR:
            self.player.set_state(gst.STATE_NULL)
            self.__stopTimer()
            modules.postMsg(modules.MSG_EVT_TRACK_ENDED)
            return True
        return False


    def play(self, filename):
        """
            Play the given file
        """
        if os.path.exists(filename):

            self.__stopTimer()

            if self.player.get_state()[1] != gst.STATE_NULL:
                self.player.set_state(gst.STATE_NULL)

            self.player.set_property('uri', 'file://' + filename)
            self.player.set_state(gst.STATE_PLAYING)
            self.__startTimer()


    def stop(self):
        """
            Stop playing
        """
        self.__stopTimer()
        self.player.set_state(gst.STATE_NULL)
        modules.postMsg(modules.MSG_EVT_STOPPED)


    def togglePause(self):
        """
            Switch between play/pause
        """
        if self.player.get_state()[1] == gst.STATE_PAUSED:
            self.player.set_state(gst.STATE_PLAYING)
            modules.postMsg(modules.MSG_EVT_UNPAUSED)
            self.__startTimer()
        elif self.player.get_state()[1] == gst.STATE_PLAYING:
            self.player.set_state(gst.STATE_PAUSED)
            modules.postMsg(modules.MSG_EVT_PAUSED)
            self.__stopTimer()


    def seek(self, seconds):
        """
            Jump to the given position
        """
        if self.player.get_state()[1] == gst.STATE_PLAYING:
            self.player.seek_simple(gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH, seconds*1000000000)


    # --== Message handler ==--


    def handleMsg(self, msg, params):
        """
            A message has been received
        """
        if   msg == modules.MSG_CMD_PLAY:         self.play(params['filename'])
        elif msg == modules.MSG_CMD_STOP:         self.stop()
        elif msg == modules.MSG_CMD_SEEK:         self.seek(params['seconds'])
        elif msg == modules.MSG_CMD_TOGGLE_PAUSE: self.togglePause()
