# -*- coding: utf-8 -*-
#
# Author: Ingelrest François (Athropos@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import modules

from threading import Lock, Semaphore, Thread


class ThreadedModule(Thread):
    """
        This is the base class for all threaded modules
    """


    def __init__(self):
        """
            Constructor
        """
        Thread.__init__(self)

        self.queue     = []              # List of queued messages
        self.mutex     = Lock()          # Protect access to the queue
        self.semaphore = Semaphore(0)    # Block the thread until some messages are available

        # Some messages required by this class
        modules.register(self, [modules.MSG_EVT_APP_QUIT])


    def postMsg(self, msg, params):
        """
            Enqueue a new message in this threads's message queue
            This function is thread-safe
        """
        self.mutex.acquire()
        self.queue.append((msg, params))
        self.mutex.release()
        self.semaphore.release()


    def dequeueMsg(self):
        """
            Dequeue the next message
            This function is thread-safe
        """
        self.mutex.acquire()
        msg = self.queue.pop(0)
        self.mutex.release()
        return msg


    def run(self):
        """
            The thread's entry point
            The execution is suspended until some messages are available
            When this happens, the main message handler is called
        """
        while True:
            self.semaphore.acquire()
            (msg, params) = self.dequeueMsg()
            self.handleMsg(msg, params)

            # The module may be unloaded for some messages
            if msg == modules.MSG_EVT_APP_QUIT:
                modules.unregister(self)
                return


    def handleMsg(self, msg, params):
        """
            This function is called when a new message is available
            It should be overridden by subclasses
        """
        pass
