# -*- coding: utf-8 -*-
#
# Author: Ingelrest François (Athropos@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import gobject, gtk, media, modules, tools

from gui            import fileChooser
from tools          import playlist
from gettext        import gettext as _
from modules.module import Module


(
    FILENAME,      # Full path to the file
    TRACKNUMBER,   # Track number
    TITLE,         # Track title
    ARTIST,        # Track artist
    ALBUM,         # Track album
    LENGTH,        # Track length in seconds
    ICON           # Icon drawn in front of the row
) = range(7)


class Tracklist(Module):
    """
        This module manages the tracklist
    """


    def __init__(self, wTree):
        """
            Constructor
        """
        Module.__init__(self)
        self.index    = None
        self.playtime = 0
        # Retrieve widgets
        self.window    = wTree.get_widget('win-main')
        self.treeview  = wTree.get_widget('tree-tracklist')
        self.btnClear  = wTree.get_widget('btn-tracklistClear')
        self.tracklist = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT, gtk.gdk.Pixbuf)
        self.treeview.set_model(self.tracklist)
        # The tracklist is empty at first
        self.btnClear.set_sensitive(False)
        # The two renderers used for the treeview
        txtRenderer = gtk.CellRendererText()
        imgRenderer = gtk.CellRendererPixbuf()
        # The icons used in the treeview
        self.nullIcon  = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 16, 16)
        self.playIcon  = self.treeview.render_icon(gtk.STOCK_MEDIA_PLAY,  gtk.ICON_SIZE_MENU, None)
        self.pauseIcon = self.treeview.render_icon(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU, None)
        self.nullIcon.fill(0x00000000)
        # Create the columns of the treeview
        self.treeview.insert_column_with_attributes(-1, '#', imgRenderer, pixbuf=ICON)
        self.treeview.get_column(0).pack_start(txtRenderer, True)
        self.treeview.get_column(0).add_attribute(txtRenderer, 'text', TRACKNUMBER)
        self.treeview.get_column(0).set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

        self.treeview.insert_column_with_attributes(-1, _('Title'),  txtRenderer, text=TITLE)
        self.treeview.get_column(1).set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.treeview.get_column(1).set_expand(True)

        self.treeview.insert_column_with_attributes(-1, _('Artist'), txtRenderer, text=ARTIST)
        self.treeview.get_column(2).set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.treeview.get_column(2).set_expand(True)

        self.treeview.insert_column_with_attributes(-1, _('Album'),  txtRenderer, text=ALBUM)
        self.treeview.get_column(3).set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.treeview.get_column(3).set_expand(True)

        self.treeview.insert_column_with_attributes(-1, _('Length'), txtRenderer, text=LENGTH)
        self.treeview.get_column(4).set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.treeview.get_column(4).set_cell_data_func(txtRenderer, lambda c, cll, mdl, it: cll.set_property('text', tools.sec2str(mdl.get_value(it, LENGTH))))
        # Messages handled by this module
        modules.register(self, [modules.MSG_CMD_TRACKLIST_SET, modules.MSG_CMD_TRACKLIST_CLEAR, modules.MSG_CMD_TRACKLIST_ADD])
        modules.register(self, [modules.MSG_CMD_TOGGLE_PAUSE, modules.MSG_CMD_NEXT, modules.MSG_CMD_PREVIOUS])
        modules.register(self, [modules.MSG_EVT_STOPPED, modules.MSG_EVT_PAUSED, modules.MSG_EVT_UNPAUSED, modules.MSG_EVT_TRACK_ENDED])
        # GTK handlers
        wTree.signal_connect('onTracklistClear',         lambda widget: modules.postMsg(modules.MSG_CMD_TRACKLIST_CLEAR))
#        wTree.signal_connect('onTracklistSaveAs',        self.onSaveAs)
        wTree.signal_connect('onTracklistAddFiles',      self.onAddFiles)
        wTree.signal_connect('onTracklistOpenFiles',     self.onOpenFiles)
        wTree.signal_connect('onTracklistMouseButton',   self.onMouseButton)
        wTree.signal_connect('onTracklistAddDirectory',  self.onAddDirectory)
        wTree.signal_connect('onTracklistOpenDirectory', self.onOpenDirectory)


    def __hasNext(self):     return self.index is not None and self.index < len(self.tracklist) - 1
    def __hasPrevious(self): return self.index is not None and self.index > 0


    def __getTrackInfo(self):
        """
            Return a tuple with track information
        """
        trackInfo = ()
        for field in range(ICON):
            trackInfo += (self.tracklist.get_value(self.tracklist.get_iter(self.index), field),)
        return trackInfo


    def append(self, files, sortTracks):
        """
            Append files to the tracklist
            If sortTracks is True, sort by ascending track number
        """
        self.treeview.freeze_child_notify()
        tracks = media.readInfo(files)
        if sortTracks:
            tracks.sort(lambda t1, t2: t1[TRACKNUMBER] - t2[TRACKNUMBER])
        for track in tracks:
            track.append(self.nullIcon)
            self.tracklist.append(track)
            self.playtime += track[LENGTH]
        self.treeview.thaw_child_notify()
        self.btnClear.set_sensitive(True)
        modules.postMsg(modules.MSG_EVT_NEW_TRACKLIST, {'files': [row[FILENAME] for row in self.tracklist], 'playtime': self.playtime})


    def set(self, files, playNow, sortTracks):
        """
            Replace the tracklist
            If sortTracks is True, sort by ascending track number
        """
        self.treeview.freeze_child_notify()
        self.tracklist.clear()
        self.playtime = 0
        self.index    = None
        self.append(files, sortTracks)
        self.treeview.thaw_child_notify()

        if playNow:
            self.startPlaying()


    def clear(self):
        """
            Clear the tracklist
        """
        self.tracklist.clear()
        self.playtime = 0
        self.index    = None
        self.btnClear.set_sensitive(False)
        modules.postMsg(modules.MSG_EVT_NEW_TRACKLIST, {'files': [], 'count': 0, 'playtime': 0})
        modules.postMsg(modules.MSG_CMD_STOP)


    def stopped(self):
        """
            Stop playing
        """
        if self.index is not None:
            self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.nullIcon)
            self.index = None


    def paused(self):
        """
            Pause playing
        """
        if self.index is not None:
            self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.pauseIcon)


    def unpaused(self):
        """
            Unpause playing
        """
        if self.index is not None:
            self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.playIcon)


    def startPlaying(self):
        """
            Start to play the first track of the playlist
        """
        self.index = 0
        self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.playIcon)
        modules.postMsg(modules.MSG_CMD_PLAY, {'filename': self.tracklist.get_value(self.tracklist.get_iter(self.index), FILENAME)})
        modules.postMsg(modules.MSG_EVT_NEW_TRACK, {'trackInfo': self.__getTrackInfo(), 'hasNext': self.__hasNext(), 'hasPrevious': self.__hasPrevious()})


    def next(self):
        """
            Jump to the next track if any
        """
        if self.__hasNext():
            self.treeview.freeze_child_notify()
            self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.nullIcon)
            self.index += 1
            self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.playIcon)
            self.treeview.thaw_child_notify()
            modules.postMsg(modules.MSG_CMD_PLAY, {'filename': self.tracklist.get_value(self.tracklist.get_iter(self.index), FILENAME)})
            modules.postMsg(modules.MSG_EVT_NEW_TRACK, {'trackInfo': self.__getTrackInfo(), 'hasNext': self.__hasNext(), 'hasPrevious': self.__hasPrevious()})


    def previous(self):
        """
            Jump to the previous track if any
        """
        if self.__hasPrevious():
            self.treeview.freeze_child_notify()
            self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.nullIcon)
            self.index -= 1
            self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.playIcon)
            self.treeview.thaw_child_notify()
            modules.postMsg(modules.MSG_CMD_PLAY, {'filename': self.tracklist.get_value(self.tracklist.get_iter(self.index), FILENAME)})
            modules.postMsg(modules.MSG_EVT_NEW_TRACK, {'trackInfo': self.__getTrackInfo(), 'hasNext': self.__hasNext(), 'hasPrevious': self.__hasPrevious()})


    def trackEnded(self):
        """
            The current track has ended, try to jump to the next one
        """
        if self.index < len(self.tracklist) - 1:
            self.next()
        else:
            modules.postMsg(modules.MSG_CMD_STOP)


    # --== Message handler ==--


    def handleMsg(self, msg, params):
        """
            A message has been received
        """
        if   msg == modules.MSG_CMD_NEXT:            self.next()
        elif msg == modules.MSG_EVT_PAUSED:          self.paused()
        elif msg == modules.MSG_EVT_STOPPED:         self.stopped()
        elif msg == modules.MSG_EVT_UNPAUSED:        self.unpaused()
        elif msg == modules.MSG_CMD_PREVIOUS:        self.previous()
        elif msg == modules.MSG_EVT_TRACK_ENDED:     self.trackEnded()
        elif msg == modules.MSG_CMD_TRACKLIST_SET:   self.set(params['files'], params['playNow'], params['sortTracks'])
        elif msg == modules.MSG_CMD_TRACKLIST_ADD:   self.append(params['files'], params['sortTracks'])
        elif msg == modules.MSG_CMD_TRACKLIST_CLEAR: self.clear()
        elif msg == modules.MSG_CMD_TOGGLE_PAUSE:
            if self.index is None:
                self.startPlaying()


    # --== GTK handlers ==--


    def onMouseButton(self, tree, event) :
        """
            A mouse button has been pressed
        """
        # Double left click
        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
            pthinfo = self.treeview.get_path_at_pos(int(event.x), int(event.y))
            if pthinfo is not None:
                if self.index is not None:
                    self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.nullIcon)
                self.index = pthinfo[0][0]
                self.tracklist.set_value(self.tracklist.get_iter(self.index), ICON, self.playIcon)
                modules.postMsg(modules.MSG_CMD_PLAY, {'filename': self.tracklist.get_value(self.tracklist.get_iter(self.index), FILENAME)})
                modules.postMsg(modules.MSG_EVT_NEW_TRACK, {'trackInfo': self.__getTrackInfo(), 'hasNext': self.__hasNext(), 'hasPrevious': self.__hasPrevious()})


    def onOpenFiles(self, widget):
        """
            Open some file(s) and replace the tracklist
        """
        files = fileChooser.openFiles(self.window, _('Select file(s)'), {'Audio files': media.getSupportedFormats()})
        if files is not None:
            modules.postMsg(modules.MSG_CMD_TRACKLIST_SET, {'files': files, 'playNow': True, 'sortTracks': True})


    def onOpenDirectory(self, widget):
        """
            Open a directory and replace the tracklist by its content
        """
        directory = fileChooser.openDirectory(self.window, _('Select a directory'))
        if directory is not None:
            mediaFiles = tools.listMediaFiles(directory)
            if mediaFiles is not None:
                modules.postMsg(modules.MSG_CMD_TRACKLIST_SET, {'files': mediaFiles, 'playNow': True, 'sortTracks': True})

    def onAddFiles(self, widget):
        """
            Open some file(s) and append them to the tracklist
        """
        files = fileChooser.openFiles(self.window, _('Select file(s)'), {'Audio files': media.getSupportedFormats()})
        if files is not None:
            modules.postMsg(modules.MSG_CMD_TRACKLIST_ADD, {'files': files, 'sortTracks': True})


    def onAddDirectory(self, widget):
        """
            Open a directory and append its content to the tracklist
        """
        directory = fileChooser.openDirectory(self.window, _('Select a directory'))
        if directory is not None:
            mediaFiles = tools.listMediaFiles(directory)
            if mediaFiles is not None:
                modules.postMsg(modules.MSG_CMD_TRACKLIST_ADD, {'files': mediaFiles, 'sortTracks': True})


    def onSaveAs(self, widget):
        """
            Save the current tracklist
        """
        file = fileChooser.save(self.window, _('Save as...'), 'playlist.m3u')
        if file is not None:
            playlist.save([row[FILENAME] for row in self.tracklist], file, False)
